import type { LanguageRegistration, ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { type FunctionComponent, useMemo } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, StyleSheet, Text, type TextStyle, View, type ViewStyle } from 'react-native'
import { getStylesFromShikiTheme } from '../../utils/style'

export interface SyntaxHighlighterProps {
  text: string
  language: string | LanguageRegistration
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
}

let highlighterInstance: Awaited<ReturnType<typeof createHighlighterCore>> | null = null

async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
    })
  }
  return highlighterInstance
}

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ text, language, theme, fontSize = 14, textStyle, scrollViewProps, containerStyle }) => {
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)

  // Convert theme to RN styles
  const stylesheet = useMemo(() => getStylesFromShikiTheme(theme), [theme])

  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true)
        const highlighter = await getHighlighter()

        // Load language and theme
        if (typeof language !== 'string') {
          await highlighter.loadLanguage(language)
        }
        await highlighter.loadTheme(theme)

        // Get the theme name from the registration
        const themeName = theme.name || Object.getOwnPropertyDescriptor(theme, 'name')?.value || 'default'

        // Highlight the code
        const result = highlighter.codeToTokens(text, {
          lang: typeof language === 'string' ? language : language.name,
          theme: themeName,
        })

        setTokens(result.tokens)
        setError(null)
      }
      catch (err) {
        setError(err instanceof Error ? err.message : 'Failed to highlight code')
        console.error('Highlighting error:', err)
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme])

  const renderToken = (token: ThemedToken, index: number) => {
    // Get token scope styles
    const scopeStyles = token.explanation?.[0]?.scopes.map(scope => stylesheet[scope.scopeName]).filter(Boolean) || []

    const style = StyleSheet.flatten([
      textStyle,
      { fontSize },
      ...scopeStyles,
      token.color ? { color: token.color } : null,
      token.fontStyle === 1 && { fontStyle: 'italic' as const },
      token.fontStyle === 2 && { fontWeight: 'bold' as const },
      token.fontStyle === 4 && { textDecorationLine: 'underline' as const },
    ])

    return (
      <Text key={`${token.offset}-${index}`} style={style}>
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => (
    <View key={index} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
      {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
    </View>
  )

  if (loading) {
    return (
      <View style={[stylesheet.root, { padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[stylesheet.root, { padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[stylesheet.root, scrollViewProps?.style]} contentContainerStyle={[containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
